import sys, os
import platform
from functools import partial
import PySide
 
from PySide.QtCore import QRect
from PySide.QtGui import QApplication, QMainWindow, QTextEdit, QPushButton,\
                         QMessageBox,  QIcon, QAction, QWidget, QGridLayout,\
                         QTextEdit, QMenuBar, QMenu, QStatusBar,\
                         QListWidget, QFrame, QComboBox, QValidator, QLineEdit,\
                         QRegExpValidator
                         

from cli import CliApp 

__version__ = '0.0.1'

class MainWindow(QMainWindow):
    def __init__(self, parent = None, config_path = None):
        super(MainWindow, self).__init__(parent)
        
        self.interface = CliApp(config_path)
        
        #window size
        self.win_width = 600
        self.win_height = 400
        self.resize(self.win_width, self.win_height)
        
        #layout right
        right_anchor = QWidget(self)
        right_anchor.resize(self.win_width/2, self.win_height)
        layout_left = QGridLayout(right_anchor)
        
        self.text = QLineEdit(self)
        self.text.setText("Path Output")
        self.text_partialfunctions = {"getpath" :  lambda qString: self.text.setText(self.getPathFromQListWidgetItem(qString))}
        #### romlist
        self.filelist = Filelist(self)
        ## filelist partial functions
        self.filelist_partialfunctions = {"refreshlist": partial(self.filelist.RefreshList, self.interface )}
        
        self.filelist.currentItemChanged.connect(self.text_partialfunctions["getpath"])
        #self.filelist.currentItemChanged.connect(self.interface.getPathFromIndex(self.text.setText))
        
        #### location bar
        self.locationbar = PySide.QtGui.QComboBox(self)
        self.locationbar.lineEdit()
        
        self.locationbar.InsertPolicy = QComboBox.InsertAtTop
        self.locationbar.setEditable(True)
        self.locationbar.setMaxCount(3)
        
        #update 
        self.filelist_partialfunctions["refreshlist"](self.locationbar.currentText())
        
        #set signals
        self.locationbar.editTextChanged.connect(self.filelist_partialfunctions["refreshlist"])
        self.locationbar.activated
        
        #cause listview to change
        self.locationbar.setEditText(self.interface.config.dict["Dir:Roms"])
        
        #launch button
        self.launchbutton_partialfunctions = {"launchsnes9x" : partial(os.popen, self.interface.snes9xbin)}
        self.launchsnes9x = QPushButton("&Launch Snes9x", self)
        self.launchsnes9x.clicked.connect(self.launchSnes9xFromQWidgetSelection)
        
        #add to right hand layout
        layout_left.addWidget(self.locationbar)
        layout_left.addWidget(self.filelist)
        layout_left.addWidget(self.text)
        layout_left.addWidget(self.launchsnes9x)
        
    def launchSnes9xFromQWidgetSelection(self, *args):
        quote = '"'
        cmd = self.interface.snes9xbin + " " + quote + self.text.text() + quote
        print cmd
        os.popen(cmd, "w")
        
    def getPathFromQListWidgetItem(self, QListWidgetItem):
        return self.interface.getPathFromName(QListWidgetItem.text())

class PathValidator(QRegExpValidator):
    def __init__(self, parent = None):
        super(PathValidator, self).__init__(parent)
    def validate(implementation, path_string):
        if implementation.checkPath(path_string):
            return QValidator.Acceptable
        else:
            return QValidator.Invalid
        
class LocationBar(QComboBox):
    def __init__(self, parent = None):
        super(LocationBar, self).__init__(parent)
        
        self.setEditText(u"Path to Roms")
        self.InsertPolicy = QComboBox.InsertAtTop
        self.setEditable(True)
        self.qobj_validator = PathValidator(self)
        self.setValidator(self.qobj_validator )
        self.setMaxCount(3)
        
class Filelist(QListWidget):
    """ scrolling list of roms """
    def __init__(self, parent = None):
        super(Filelist, self).__init__(parent)
        self.setUniformItemSizes(True)
        
        self.resize(parent.win_width/2, parent.win_height/1.2)
    
    def RefreshList(self, interface, path_string):
        #can return a list of names or paths or a dict of both 
        listing = interface.listDirectory(path_string)[0]
        self.clear()
        self.addItems(listing)        
        
class InfoView(QFrame):
    """ rom meta info """
    def __init__(self, parent = None):
        super(RomView, self).__init__(parent)
        
if __name__ == '__main__':
    app = QApplication(sys.argv)
    if os.name == "nt":
        #windows
        path = os.path.join(__file__[:__file__.rfind(os.path.sep)], "snes9x.conf")
    elif os.name == "posix":
        #linux
        path = os.path.expanduser("~/.snes9x/snes9x.config")
        
    frame = MainWindow(None, path)
    frame.show()
    sys.exit(app.exec_())